from typing import Any, List, Union
import pandas as pd
from datamaxi.api import API
from datamaxi.lib.utils import check_required_parameters
from datamaxi.lib.utils import postprocess
from datamaxi.lib.constants import BASE_URL


class Binance(API):
    """Client to fetch Binance data from DataMaxi+ API."""

    def __init__(self, api_key=None, **kwargs: Any):
        """Initialize the object.

        Args:
            api_key (str): The DataMaxi+ API key
            **kwargs: Keyword arguments used by `datamaxi.api.API`.
        """
        if "base_url" not in kwargs:
            kwargs["base_url"] = BASE_URL

        super().__init__(api_key, **kwargs)

    def symbols(self) -> List[str]:
        """Binance supported symbols

        `GET /v1/raw/binance/symbols`

        <https://docs.datamaxiplus.com/api/datasets/cex-raw/binance/symbols>

        Returns:
            List of supported Binance symbols
        """
        url_path = "/v1/raw/binance/symbols"
        return self.query(url_path)

    def intervals(self) -> List[str]:
        """Binance supported intervals

        `GET /v1/raw/binance/intervals`

        <https://docs.datamaxiplus.com/api/datasets/cex-raw/binance/intervals>

        Returns:
            List of supported Binance intervals
        """
        url_path = "/v1/raw/binance/intervals"
        return self.query(url_path)

    @postprocess()
    def candle(
        self,
        symbol: str,
        interval: str = "1d",
        market: str = "spot",
        pandas: bool = True,
    ) -> Union[List, pd.DataFrame]:
        """Get Binance candle data

        `GET /v1/raw/binance/candle`

        <https://docs.datamaxiplus.com/api/datasets/cex-raw/binance/candle>

        Args:
            symbol (str): Binance symbol
            interval (str): Candle interval
            market (str): Market type (spot/futures)
            pandas (bool): Return data as pandas DataFrame

        Returns:
            Binance candle data for a given symbol, interval and market in pandas DataFrame
        """
        check_required_parameters(
            [[symbol, "symbol"], [interval, "interval"], [market, "market"]]
        )

        if market not in ["spot", "futures"]:
            raise ValueError("market must be either spot or futures")

        params = {"symbol": symbol, "interval": interval, "market": market}
        return self.query("/v1/raw/binance/candle", params)



from typing import List
from urllib.parse import urlencode
import pandas as pd
from functools import wraps
from datamaxi.error import ParameterRequiredError
from datamaxi.error import AtLeastOneParameterRequiredError


def to_float(x):
    return float(x)


def to_int(x):
    return int(x)


def cleanNoneValue(d) -> dict:
    out = {}
    for k in d.keys():
        if d[k] is not None:
            out[k] = d[k]
    return out


def check_required_parameter(value, name: str):
    if not value and value != 0:
        raise ParameterRequiredError([name])


def check_required_parameters(params):
    """Validate multiple parameters
    params = [
        ['btcusdt', 'symbol'],
        [10, 'price']
    ]

    """
    for p in params:
        check_required_parameter(p[0], p[1])


def check_at_least_one_set_parameters(params):
    at_least_one_set = False
    for p in params:
        try:
            check_required_parameter(p[0], p[1])
            at_least_one_set = True
            break
        except:  # noqa: E722
            pass

    if not at_least_one_set:
        raise AtLeastOneParameterRequiredError()


def check_required_parameter_list(values: List, name: str):
    if len(values) == 0:
        raise ParameterRequiredError([name])

    for value in values:
        check_required_parameter(value, name)


def encode_string_list(L: List):
    return "[" + ",".join(f'"{i}"' for i in L) + "]"


def encoded_string(query):
    return urlencode(query, True).replace("%40", "@")


def convert_to_df(data, header: bool, index: str = None, apply_fn={}):
    df = pd.DataFrame(data)

    if header:
        new_header = df.iloc[0]
        df = df[1:]
        df.columns = new_header

    if index is not None:
        df = df.set_index(index)

    for idx, fn in apply_fn.items():
        if isinstance(idx, str):
            df[idx] = df[idx].apply(fn)
        else:
            df[df.columns[idx]] = df[df.columns[idx]].apply(fn)

    return df


def make_list(value):
    if not isinstance(value, list):
        value = [value]
    return value


def _postprocess(data, index):
    apply_fn = {}
    for idx in range(len(data[0]) - len(index)):
        apply_fn[idx] = to_float

    return convert_to_df(data, header=True, index=index, apply_fn=apply_fn)


def postprocess(num_index: int = 1):
    def decorator(func):
        @wraps(func)
        def wrapper(*arg, **kwarg):
            res = func(*arg, **kwarg)

            if not kwarg.get("pandas", True):
                return res

            if isinstance(res, dict):
                res["data"] = _postprocess(res["data"], res["data"][0][:num_index])
            else:
                res = _postprocess(res, res[0][:num_index])

            return res

        return wrapper

    return decorator



from typing import Any, List, Union
import pandas as pd
from datamaxi.api import API
from datamaxi.lib.utils import check_required_parameter
from datamaxi.lib.utils import check_required_parameters
from datamaxi.lib.utils import postprocess
from datamaxi.lib.constants import BASE_URL


class Datamaxi(API):
    """Client to fetch unified data from DataMaxi+ API."""

    def __init__(self, api_key=None, **kwargs: Any):
        """Initialize the object.

        Args:
            api_key (str): The DataMaxi+ API key
            **kwargs: Keyword arguments used by `datamaxi.api.API`.
        """
        if "base_url" not in kwargs:
            kwargs["base_url"] = BASE_URL

        super().__init__(api_key, **kwargs)

    def symbols(self, exchange: str) -> List[str]:
        """Supported symbols by given exchange

        `GET /v1/symbols`

        <https://docs.datamaxiplus.com/api/datasets/cex/symbols>

        Args:
            exchange (str): Exchange name

        Returns:
            List of supported symbols
        """
        check_required_parameter(exchange, "exchange")
        params = {"exchange": exchange}
        url_path = "/v1/symbols"
        return self.query(url_path, params)

    def intervals(self, exchange: str) -> List[str]:
        """Supported intervals by given exchange

        `GET /v1/intervals`

        <https://docs.datamaxiplus.com/api/datasets/cex/intervals>

        Args:
            exchange (str): Exchange name

        Returns:
            List of supported intervals
        """
        check_required_parameter(exchange, "exchange")
        params = {"exchange": exchange}
        url_path = "/v1/intervals"
        return self.query(url_path, params)

    @postprocess()
    def candle(
        self,
        exchange: str,
        symbol: str,
        interval: str = "1d",
        market: str = "spot",
        pandas: bool = True,
    ) -> Union[List, pd.DataFrame]:
        """Get candle data

        `GET /v1/candle`

        <https://docs.datamaxiplus.com/api/datasets/cex/candle>

        Args:
            exchange (str): Exchange name
            symbol (str): Symbol name
            interval (str): Candle interval
            market (str): Market type (spot/futures)
            pandas (bool): Return data as pandas DataFrame

        Returns:
            Candle data for a given symbol, interval and market in pandas DataFrame
        """
        check_required_parameters(
            [
                [exchange, "exchange"],
                [symbol, "symbol"],
                [interval, "interval"],
                [market, "market"],
            ]
        )

        if market not in ["spot", "futures"]:
            raise ValueError("market must be either spot or futures")

        params = {
            "exchange": exchange,
            "symbol": symbol,
            "interval": interval,
            "market": market,
        }
        return self.query("/v1/candle", params)



from typing import Any, List, Union
import pandas as pd
from datamaxi.api import API
from datamaxi.lib.utils import check_required_parameters
from datamaxi.lib.utils import postprocess
from datamaxi.lib.constants import BASE_URL


class Gopax(API):
    """Client to fetch Gopax data from DataMaxi+ API."""

    def __init__(self, api_key=None, **kwargs: Any):
        """Initialize the object.

        Args:
            api_key (str): The DataMaxi+ API key
            **kwargs: Keyword arguments used by `datamaxi.api.API`.
        """
        if "base_url" not in kwargs:
            kwargs["base_url"] = BASE_URL

        super().__init__(api_key, **kwargs)

    def symbols(self) -> List[str]:
        """Gopax supported symbols

        `GET /v1/raw/gopax/symbols`

        <https://docs.datamaxiplus.com/api/datasets/cex-raw/gopax/symbols>

        Returns:
            List of supported Gopax symbols
        """
        url_path = "/v1/raw/gopax/symbols"
        return self.query(url_path)

    def intervals(self) -> List[str]:
        """Gopax supported intervals

        `GET /v1/raw/gopax/intervals`

        <https://docs.datamaxiplus.com/api/datasets/cex-raw/gopax/intervals>

        Returns:
            List of supported Gopax intervals
        """
        url_path = "/v1/raw/gopax/intervals"
        return self.query(url_path)

    @postprocess()
    def candle(
        self, symbol: str, interval: str = "1d", pandas: bool = True
    ) -> Union[List, pd.DataFrame]:
        """Get Gopax candle data

        `GET /v1/raw/gopax/candle`

        <https://docs.datamaxiplus.com/api/datasets/cex-raw/gopax/candle>

        Args:
            symbol (str): Gopax symbol
            interval (str): Candle interval
            pandas (bool): Return data as pandas DataFrame

        Returns:
            Gopax candle data for a given symbol and interval in pandas DataFrame
        """
        check_required_parameters([[symbol, "symbol"], [interval, "interval"]])
        params = {"symbol": symbol, "interval": interval}
        return self.query("/v1/raw/gopax/candle", params)
